@@ -40,7 +40,6 @@
 #include "database_p.h"
 
 static retvalue files_get_checksums(struct database *database, const char *filekey, /*@out@*/struct checksums **checksums_p) {
-	const char *md5sum = NULL;
 	const char *checksums;
 	size_t checksumslen;
 	retvalue r;
@@ -49,7 +48,7 @@ static retvalue files_get_checksums(struct database *database, const char *filek
 		&checksums, &checksumslen);
 	if( !RET_IS_OK(r) )
 		return r;
-	return checksums_setall(checksums_p, checksums, checksumslen, md5sum);
+	return checksums_setall(checksums_p, checksums, checksumslen);
 }
 
 retvalue files_add_checksums(struct database *database, const char *filekey, const struct checksums *checksums) {
@@ -224,6 +223,40 @@ retvalue files_expectfiles(struct database *database, const struct strlist *file
 	return RET_OK;
 }
 
+/* check for several files in the database and update information */
+retvalue files_checkorimprove(struct database *database, const struct strlist *filekeys, struct checksums *checksumsarray[]) {
+	int i;
+	retvalue r;
+
+	for( i = 0 ; i < filekeys->count ; i++ ) {
+		const char *filekey = filekeys->values[i];
+		const struct checksums *checksums = checksumsarray[i];
+		struct checksums *indatabase;
+		bool improves;
+
+		r = files_get_checksums(database, filekey, &indatabase);
+		if( r == RET_NOTHING ) {
+			fprintf(stderr, "Missing file %s\n", filekey);
+			return RET_ERROR_MISSING;
+		}
+		if( RET_WAS_ERROR(r) )
+			return r;
+		if( !checksums_check(checksums, indatabase, &improves) ) {
+			fprintf(stderr,
+"File \"%s\" is already registered with different checksums!\n",
+				filekey);
+			checksums_printdifferences(stderr, indatabase, checksums);
+			r = RET_ERROR_WRONG_MD5;
+		} else if( improves ) {
+			r = checksums_combine(&checksumsarray[i], indatabase, NULL);
+		}
+		checksums_free(indatabase);
+		if( RET_WAS_ERROR(r) )
+			return r;
+	}
+	return RET_OK;
+}
+
 /* print missing files */
 retvalue files_printmissing(struct database *database, const struct strlist *filekeys, const struct checksumsarray *origfiles) {
 	int i;
@@ -368,8 +401,7 @@ retvalue files_checkpool(struct database *database, bool fast) {
 		return r;
 	while( cursor_nexttempdata(database->checksums, cursor,
 				&filekey, &combined, &combinedlen) ) {
-		r = checksums_setall(&expected,
-				combined, combinedlen, NULL);
+		r = checksums_setall(&expected, combined, combinedlen);
 		if( RET_WAS_ERROR(r) ) {
 			RET_UPDATE(result, r);
 			continue;
@@ -415,7 +447,7 @@ retvalue files_collectnewchecksums(struct database *database) {
 		return r;
 	while( cursor_nexttempdata(database->checksums, cursor,
 				&filekey, &all, &alllen) ) {
-		r = checksums_setall(&expected, all, alllen, NULL);
+		r = checksums_setall(&expected, all, alllen);
 		if( !RET_IS_OK(r) ) {
 			RET_UPDATE(result, r);
 			continue;
@@ -794,3 +826,16 @@ retvalue files_checkincludefile(struct database *database, const char *sourcedir
 
 	return files_add_checksums(database, filekey, *checksums_p);
 }
+
+off_t files_getsize(struct database *database, const char *filekey) {
+	retvalue r;
+	off_t s;
+	struct checksums *checksums;
+
+	r = files_get_checksums(database, filekey, &checksums);
+	if (!RET_IS_OK(r))
+		return -1;
+	s = checksums_getfilesize(checksums);
+	checksums_free(checksums);
+	return s;
+}
